TERMINOLOGY = {
    'ammunition': ['ammunition'],
    'armour': ['clothing'],
    'attack_damage': ['injuries'],
    'enemy': ['zombies'],
    'fuel': ['generator fuel'],
    'medicine': ['antidote'],
    'passive_damage': ['z-virus'],
    'supplies': ['food'],
    'currency': ['inventory slots']
}

RANDOM_EVENTS = {
    'attack': {'conclusion': ["just barely, but you managed to defeat it",
                              "headshot!",
                              "solved with an axe to the head"],
               'damage': 'received damage',
               'die': '',
               'error': 'you trapped the zombie, but are are now running out of ammunition',
               'events': [],
               'intro': 'a zombie sneaks on you'},
    'illness': {'conclusion': '',
                'damage': '',
                'die': '',
                'error': '',
                'events': [],
                'intro': 'you are starving'},
    'poison': {'conclusion': '',
               'die': 'you die since you have no medicine',
               'error': '',
               'events': [],
               'intro': 'you ate infected food'},
    'animal': {'conclusion': 'scared them away',
               'damage': 'got bitten',
               'die': '',
               'error': ["lost a boot", "lost clothing", "lost knife", "lost food", "damaged jacket"],
               'events': [],
               'intro': 'rabid dogs attack!'},
    'companion': {'conclusion': '',
                  'damage': '',
                  'die': '',
                  'error': '',
                  'events': [],
                  'intro': ['you spend time making zombie traps',
                            'a zombie managed to bypass your barricade, you killed and spent time looking for the hole',
                            'barbed wire fence maintenance']},
    'bad_terrain': {'conclusion': 'you use food and clothes to make a decoy and lure them away',
                    'intro': 'a zombie horde is gathering outside'},
    'companion_lose': {'conclusion': 'spent time repairing it',
                       'intro': 'the barricade sustained damage'},
    'find_supplies': {'conclusion': '', 'intro': 'found supplies'},
    'fuel_damage': {'conclusion': 'spend time scavenging for pieces',
                    'intro': 'generator problems'},
    'shelter_damage': {'conclusion': 'lose time and supplies fixing it',
                       'intro': ['shelter damage', 'barricade suffered damage']},
    'shelter_fire': {'conclusion': 'supplies damaged',
                     'intro': ['kitchen fire']},
    'supply_damage': {'conclusion': 'lose time scavenging for more',
                      'intro': 'supplies perished'},
    'vehicle_damage': {'conclusion': 'lose time, you must stay at home until they leave',
                       'intro': 'zombie swarm at the door'},
    'rain': {'conclusion': 'time and supplies lost',
             'intro': 'rain'
             },
    'storm': {'conclusion': 'supplies damaged', 'intro': 'storm'},
    'fog': {'conclusion': 'time is lost',
            'intro': 'lose your way in heavy fog'},
    'cold': {'conclusion': 'you have enough clothing',
             'error': 'frostbite',
             'intro': 'cold'}
}

GAME_EVENTS = {
    'damage': {'conclusion': '',
               'error': 'you ran out medical supplies\nyou died of',
               'high': 'critical health --medicine used',
               'intro': '',
               'mild': 'low health ---medicine used'},
    'difficulty_damage': {'conclusion': '',
                          'error': '',
                          'events': [],
                          'intro': 'time and supplies lost'},
    'easy_difficulty': {'intro': 'you made it safely to the lab',
                        'conclusion': 'you have no idea how to break in',
                        'events': ['you got lost---lose valuable time trying to '
                                   'find trail!',
                                   'vehicle damaged!---lose time and supplies',
                                   'dangerous path ahead']},
    'enemy_encounter': {'conclusion': '',
                        'damage': 'sustained damage',
                        'die': 'got killed by ',
                        'enemy_run': 'they did not attack',
                        'high_score': 'you killed them all',
                        'hostile_conclusion': ' attacked you',
                        'hostile_intro': 'they look hostile',
                        'intro': ' ahead.',
                        'low_score': 'kinda slow with your shooting',
                        'number_questions': ['(1) run  (2) attack  (3) continue  '
                                             '(4) circle'],
                        'peaceful_conclusion': ' couldn\'t reach you',
                        'peaceful_intro': "they appear crippled"},
    'explore': {'conclusion': '',
                'error': 'can not explore without ammunition',
                'events': ['found supplies!',
                           'sorry---no luck today',
                           'found a lot of supplies'],
                'intro': ''},
    'game_over': {'conclusion': 'distance to lab: ',
                  'error': 'time has run out',
                  'intro': 'you never found the cure'},
    'hard_difficulty': {'conclusion': 'violent thoughts plague your mind',
                        'intro': 'You notice a mirror, your eyes are turning red'},
    'heal': {'conclusion': "you prepare 20 vials of virus retardant",
             'die': 'you died of ',
             'error': "you are out of antidote",
             'events': [],
             'intro': ''},
    'intro': {
        'conclusion': 'you hear rumours about a cure being successfully developed in a lab just before all communication broke down...\nfind the cure before you turn',
        'intro': 'It\'s been 1 year since the outbreak, yesterday you got bitten, damn zombie found a hole in your defences...'},
    'inventory': {'conclusion': 'after all your choices, you now have '
                                '{inv.money} slots left',
                  'error': 'you overshoot, you only had {inv.start_money} slots to '
                           'spend.  chose again',
                  'intro': 'how many slots do you want to assign to '},
    'lose': {'intro': 'you died, just before you turned you saw your first face in months, you tried to say something',
             "conclusion": "brains..."},
    'low_supplies': {'intro': 'low supplies'},
    'maintenance': {'conclusion': '',
                    'error': 'not enough supplies',
                    'intro': 'consume supplies (1) a little  (2) moderately (3) a lot'},
    'medium_difficulty': {'conclusion': 'you begin to wonder if the cure is real', 'intro': 'You break into the lab'},
    'numeric': {'error': 'impossible', 'high': 'too high', 'low': 'too low'},
    'shop': {'conclusion': '',
             'error': "you don't have that much slots your spending down",
             'events': [],
             'intro': 'what are we scavenging for'},
    'turn': {'die': 'you ran out of supplies and starved to death',
             'intro': '(1) go scavenging, (2) explore, (3) travel'},
'win': {'intro': 'you made it to the lab\nyou found the cure on time, however you are no scientist\nyou cure yourself and use every last bit of the ZVirus cure\n you realize the world is still full of zombies...\nhopefully you are now immune, happy zombie hunting!'}}
