from setuptools import setup

setup(
    name='TNaLaGmes',
    version='0.1.2',
    packages=['tnalagmes', 'tnalagmes.data', 'tnalagmes.games', 'tnalagmes.engines',
              'tnalagmes.models', 'tnalagmes.lang', 'tnalagmes.locale', 'tnalagmes.util'],
    url='https://github.com/JarbasAl/TNaLaGmes',
    license='Apache2.0',
    author='jarbasAI',
    author_email='jarbasai@mailfence.com',
    description='TNaLaGmes is a Toolbox for Natural Language Games'
)
