TERMINOLOGY = {
    'ammunition': ['ammunition'],
    'armour': ['armour'],
    'attack_damage': ['injuries'],
    'currency': ['cash'],
    'enemy': ['strangers'],
    'fuel': ['fuel'],
    'medicine': ['medicine'],
    'passive_damage': ['disease'],
    'supplies': ['supplies']}

event_template = {
    "intro": "",
    "events": [],
    "conclusion": "",
    "error": "",
    "damage": "",
    "die": ""
}

RANDOM_EVENTS = {

    'attack': {'conclusion': 'enemies defeated',
               'damage': 'received damage',
               'die': '',
               'error': 'out of ammunition',
               'events': [],
               'intro': 'enemies attack'},
    'illness': {'conclusion': '',
                'damage': '',
                'die': '',
                'error': '',
                'events': [],
                'intro': ''},
    'poison': {'conclusion': '',
               'die': 'you die since you have no medicine',
               'error': '',
               'events': [],
               'intro': 'ate poisonous food'},
    'animal': {'conclusion': 'scared them away',
               'damage': 'got bitten',
               'die': '',
               'error': 'lost armour',
               'events': [],
               'intro': 'wild animals attack!'},
    'companion': {'conclusion': '',
                  'damage': '',
                  'die': '',
                  'error': '',
                  'events': [],
                  'intro': 'companion got injured'},

    'bad_terrain': {'conclusion': 'lose food and clothes',
                    'intro': 'bad terrain causes damage'},
    'companion_lose': {'conclusion': 'spend half the day looking for '
                                     'him',
                       'intro': 'your companion got lost'},
    'find_supplies': {'conclusion': '', 'intro': 'found supplies'},
    'fuel_damage': {'conclusion': 'spend time looking for it',
                    'intro': 'vehicle stolen'},
    'shelter_damage': {'conclusion': 'lose time and supplies fixing '
                                     'it',
                       'intro': 'shelter damage'},
    'shelter_fire': {'conclusion': 'supplies damaged',
                     'intro': 'shelter fire'},
    'supply_damage': {'conclusion': 'lose time looking for more',
                      'intro': 'supplies perished'},
    'vehicle_damage': {'conclusion': 'slows you down rest of trip',
                       'intro': 'vehicle_damage'}
    ,

    'rain': {'conclusion': 'time and supplies lost',
             'intro': 'rain'
             },
    'storm': {'conclusion': 'supplies damaged', 'intro': 'storm'},
    'fog': {'conclusion': 'time is lost',
            'intro': 'lose your way in heavy fog'},
    'cold': {'conclusion': 'you have enough clothing',
             'error': 'frostbite',
             'intro': 'cold'}
}

GAME_EVENTS = {
    'damage': {'conclusion': '',
               'error': 'you ran out medical supplies\nyou died of',
               'high': 'critical health --medicine used',
               'intro': '',
               'mild': 'low health ---medicine used'},
    'difficulty_damage': {'conclusion': '',
                          'error': '',
                          'events': [],
                          'intro': 'time and supplies lost'},
    'easy_difficulty': {'conclusion': 'you made it safely',
                        'events': ['you got lost---lose valuable time trying to '
                                   'find trail!',
                                   'vehicle damaged!---lose time and supplies',
                                   'dangerous path ahead']},
    'enemy_encounter': {'conclusion': '',
                        'damage': 'sustained damage',
                        'die': 'got killed by ',
                        'enemy_run': 'they did not attack',
                        'high_score': 'you drove them off',
                        'hostile_conclusion': ' were hostile',
                        'hostile_intro': 'they look hostile',
                        'intro': ' ahead.',
                        'low_score': 'kinda slow with your shooting',
                        'number_questions': ['(1) run  (2) attack  (3) continue  '
                                             '(4) circle'],
                        'peaceful_conclusion': ' were friendly',
                        'peaceful_intro': "they don't look hostile"},
    'explore': {'conclusion': '',
                'error': 'can not explore without ammunition',
                'events': ['found supplies!',
                           'sorry---no luck today',
                           'found a lot of supplies'],
                'intro': ''},
    'game_over': {'conclusion': 'distance remaining: ',
                  'error': 'time has run out',
                  'intro': ''},
    'hard_difficulty': {'conclusion': '', 'intro': ''},
    'heal': {'conclusion': "doctor's bill is $20",
             'die': 'you died of ',
             'error': "you can't afford a doctor",
             'events': [],
             'intro': ''},
    'intro': {'conclusion': 'try not to die', 'intro': 'travel to safety'},
    'inventory': {'conclusion': 'after all your purchases, you now have '
                                '{inv.money} dollars left',
                  'error': 'you overspent--you only had $ {inv.start_money} to '
                           'spend.  buy again',
                  'intro': 'how much do you want to spend on '},
    'lose': {'intro': 'you died', "conclusion": ""},
    'low_supplies': {'intro': 'low supplies'},
    'maintenance': {'conclusion': '',
                    'error': 'not enough supplies',
                    'intro': 'consume supplies (1) a little  (2) moderately (3) a lot'},
    'medium_difficulty': {'conclusion': 'you made it safely', 'intro': ''},
    'numeric': {'error': 'impossible', 'high': 'too high', 'low': 'too low'},
    'shop': {'conclusion': '',
             'error': "you don't have that much--keep your spending down",
             'events': [],
             'intro': 'enter what you wish to spend on the following'},
    'turn': {'die': 'you ran out of supplies and starved to death',
             'intro': '(1) shop, (2) explore, (3) travel'},
    'win': {'intro': 'you survived'}}
